#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_fs_1 = __importDefault(require("node:fs"));
const node_path_1 = __importDefault(require("node:path"));
const yargs_1 = __importDefault(require("yargs"));
const helpers_1 = require("yargs/helpers");
const chalk_1 = __importDefault(require("chalk"));
const cli_spinners_1 = __importDefault(require("cli-spinners"));
const log_update_1 = __importDefault(require("log-update"));
const prompts_1 = __importDefault(require("prompts"));
const file_writers_1 = require("./templates/file-writers");
// @ts-ignore
const package_json_1 = require("../package.json");
const argv = (0, yargs_1.default)((0, helpers_1.hideBin)(process.argv)).argv;
const cwd = process.cwd();
var ArgName;
(function (ArgName) {
    ArgName["type"] = "type";
    ArgName["name"] = "name";
    ArgName["lang"] = "lang";
    ArgName["port"] = "port";
    ArgName["overwrite"] = "overwrite";
})(ArgName || (ArgName = {}));
const DeckTypeOptions = [
    { title: chalk_1.default.cyan('tsx'), value: 'tsx' },
    { title: chalk_1.default.yellow('jsx'), value: 'jsx' },
    // { title: chalk.red('mdx'), value: 'mdx' },
    { title: chalk_1.default.green('One Page'), value: 'onepage' }
];
let progressInterval;
const log = console.log;
const printConsoleError = (message) => chalk_1.default.whiteBright.bgRed.bold(' ! ') + chalk_1.default.red.bold(' ' + message + '\n');
const sleep = (ms) => new Promise((resolve) => setTimeout(resolve, ms));
const main = async () => {
    log(chalk_1.default.whiteBright.bgMagenta.bold(' Spectacle CLI '));
    let i = 0;
    let type = argv[ArgName.type] || argv['t'];
    let name = argv['_']?.[0];
    let lang = argv[ArgName.lang] || argv['l'] || 'en';
    let port = argv[ArgName.port] || argv['p'] || 3000;
    const isTryingToOverwrite = Boolean(name) && !isFolderNameAvailable(name);
    /**
     * If type/name not both provided via CLI flags, prompt for them.
     */
    const hasType = Boolean(type);
    const hasName = Boolean(name);
    const hasLang = Boolean(lang);
    const hasPort = type === 'onepage' || Boolean(port); // onepage has no port
    if (!(hasType && hasName && hasLang && hasPort) || isTryingToOverwrite) {
        try {
            const response = await (0, prompts_1.default)([
                // Name prompt
                {
                    type: 'text',
                    name: ArgName.name,
                    message: 'What is the name of the presentation?',
                    initial: name,
                    validate: async (val) => {
                        return val.trim().length > 0 ? true : 'Name is required';
                    }
                },
                // If output directory already exists, prompt to overwrite
                {
                    type: (val) => (isFolderNameAvailable(val) ? null : 'confirm'),
                    name: ArgName.overwrite,
                    message: (val) => `Target directory ${formatProjectDirName(val)} already exists. Overwrite and continue?`
                },
                // Check overwrite comes back false, we need to abort.
                {
                    type: (_, answers) => {
                        if (answers?.[ArgName.overwrite] === false) {
                            throw new Error('❌ Operation cancelled');
                        }
                        return null;
                    },
                    name: 'overwriteAborter'
                },
                {
                    type: 'select',
                    name: ArgName.type,
                    message: 'What type of deck do you want to create?',
                    choices: DeckTypeOptions,
                    initial: (() => {
                        const ind = DeckTypeOptions.findIndex((o) => o.value === type);
                        return ind > -1 ? ind : 0;
                    })()
                },
                // Language prompt
                {
                    type: 'text',
                    name: ArgName.lang,
                    message: 'What is the language code for the generated HTML document?',
                    initial: lang,
                    validate: async (val) => {
                        return val.trim().length > 0 ? true : 'Language code is required';
                    }
                },
                {
                    // Don't prompt for this if onepage
                    type: (_, answers) => answers?.[ArgName.type] === 'onepage' ? null : 'text',
                    name: ArgName.port,
                    message: 'What port should the webpack dev server run on?',
                    initial: port
                }
            ], {
                onCancel: () => {
                    throw new Error('❌ Operation cancelled');
                }
            });
            if (response.type)
                type = response.type;
            if (response.name)
                name = response.name;
            lang = response.lang;
            port = response.port;
        }
        catch (err) {
            console.log(chalk_1.default.red(err.message));
            return;
        }
    }
    progressInterval = setInterval(() => {
        const { frames } = cli_spinners_1.default.aesthetic;
        (0, log_update_1.default)(chalk_1.default.whiteBright.bgBlue.bold(` ${frames[(i = ++i % frames.length)]} `), chalk_1.default.blue.bold('Building Deck'));
    }, cli_spinners_1.default.dots.interval);
    await sleep(750);
    const fileOptions = {
        snakeCaseName: formatProjectDirName(name),
        name,
        lang,
        port,
        enableTypeScriptSupport: type === 'tsx',
        spectacleVersion: package_json_1.devDependencies.spectacle
    };
    switch (type) {
        case 'jsx':
            await (0, file_writers_1.writeWebpackProjectFiles)(fileOptions);
            break;
        case 'tsx':
            await (0, file_writers_1.writeWebpackProjectFiles)(fileOptions);
            break;
        case 'onepage':
            await (0, file_writers_1.writeOnePageHTMLFile)(fileOptions);
            break;
    }
    clearInterval(progressInterval);
    (0, log_update_1.default)(chalk_1.default.whiteBright.bgGreen.bold(` \u2714 `), chalk_1.default.green.bold(`A new ${type} deck named ${name} was created.\n`));
};
const formatProjectDirName = (name) => name.toLowerCase().replace(/([^a-z0-9]+)/gi, '-');
const isFolderNameAvailable = (name) => {
    const dir = node_path_1.default.join(cwd, formatProjectDirName(name));
    return !node_fs_1.default.existsSync(dir);
};
main().catch((err) => {
    clearInterval(progressInterval);
    (0, log_update_1.default)(printConsoleError(err.message));
});
